/**

Copyright (C) 2004  Davide Carboni

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

**/


function GnutellaSocket(host,port){
	s=new Socket(host,port,true);
	
	s.responseCode="";
	
	s.httpHeaders=[];
	
	s.parseResponse=function(response){
		tokens=response.split("\n");
		gnutellaMessage=tokens.shift();
		gnutellaMessageTokens=gnutellaMessage.split(" ");
		this.responseCode=gnutellaMessageTokens[1];
		for(i=0;i<tokens.length;i++){
			this.httpHeaders[tokens[i].split(": ")[0]]=tokens[i].split(": ")[1];
		}
		
		
	}
	
	s.connect=function(){
		this.hello();
		pause(1000);
		response=this.get();
		dump(response);
		this.parseResponse(response);
		pause(1000);
		this.ack();
		
		return response;
	}

	s.pong=function(){
		dump("sending Pong\n");
		this.send(new Pong());
	}
	
	s.query=function(queryString){
		dump("sending Query: "+queryString+"\n");
		q=new Query(queryString)
		this.send(q);
		return q.UUID;
	}
	
	s.hello=function(){
		str="GNUTELLA CONNECT/0.6\n";
		str+="User-Agent: socket.js/1.0\n";
		str+="X-Ultrapeer: False\n\n";
		dump(str);
		this.write(str);
	}
	
	s.get=function(){
		k=this.available();
		string="";
		for(i=0;i<k;i++)
			string=string+String.fromCharCode(this.readByte());
		dump(string);
		return string;
	
		
	}
	
	s.ack=function(){
		str="GNUTELLA/0.6 200 OK\n\n";
		dump(str);
		this.write(str);
	}	
	
	s.ping=function(){
		p=new Ping();
		dump("sending ping");
		dump(p);
		this.send(p);
	}
	
	s.send=function(packet){
		bytes=packet.getBytes();
		for(i=0;i<bytes.length;i++)
		    this.writeByte(bytes[i]);
	}
	
	s.isConnected=function(){
		return (this.responseCode == "200");
	}
	return s;
}




function GnutellaPacket(UUID,type,TTL,hops,payload){
	this.UUID=UUID;
	this.UUID[8]=0xff;
	this.UUID[15]=0x00;
	this.type=type;
	this.TTL=TTL;
	this.hops=hops;
	this.payload=payload;
	this.psize=this.plength(payload);
}


GnutellaPacket.prototype.getBytes=function(){
	bytes=[];
	bytes=bytes.concat(this.UUID);
	bytes.push(this.type);
	bytes.push(this.TTL);
	bytes.push(this.hops);
	bytes=bytes.concat(this.psize);
	bytes=bytes.concat(this.payload);
	return bytes;
}

GnutellaPacket.prototype.unmarshall=function(binarystream){
	for(i=0;i<16;i++)
		this.UUID[i]=binarystream.read8();

	dump("\nunmarshall:UUID="+this.UUID);
	this.type=binarystream.read8();
	dump("\nunmarshall:type="+this.type);
	this.TTL=binarystream.read8();
	dump("\nunmarshall:TTL="+this.TTL);
	this.hops=binarystream.read8();
	dump("\nunmarshall:hops="+this.hops);
	this.psize[0]=binarystream.read8();
	this.psize[1]=binarystream.read8();
	this.psize[2]=binarystream.read8();
	this.psize[3]=binarystream.read8();
	
	a=this.psize;
	dump("\nunmarshall:size array="+a);
	size=a[0] + a[1]*0x100 + a[2] * 0x10000 + a[3] * 0x1000000;
	dump("\nunmarshall:size="+size);
	this.payload=[];
	for(i=0;i<size;i++)
		this.payload.push(binarystream.read8());
	dump("\nunmarshall:"+this.payload.length+" bytes read!!\n\n");
}

GnutellaPacket.prototype.toString=function(){
	s="";
	s+="\nUUID="+this.UUID;
	s+="\ntype="+this.type;
	s+="\nTTL="+this.TTL;
	s+="\nhops="+this.hops;
	s+="\npsize="+this.psize;
	s+="\npayload=["+this.payload+"]";
	return s;
}

GnutellaPacket.prototype.plength=function(payload){
		N=payload.length;
		return toLittleEndian(N,4);
}

function Ping(){
	return new GnutellaPacket(uuid(16),0x00,0x50,0x00,[]);
} 

function Pong(){
	return new GnutellaPacket(uuid(16),0x01,0x7,0x00,[0x50,0x50,0xa0,0xa0,0x48,0x48,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00]);
} 


function Query(queryString){
	q=new GnutellaPacket(uuid(16),0x80,0x7,0x00,prepareQueryPayload(queryString));
	return q;
}

function QueryHit(byteArray){
	this.numberOfHits=pop8(byteArray);
	this.port=pop16(byteArray);
	this.IP=pop8(byteArray)+"."+pop8(byteArray)+"."+pop8(byteArray)+"."+pop8(byteArray);
	this.speed=pop32(byteArray);
	this.resultSet=new ResultSet(this.numberOfHits,byteArray);
}


function prepareQueryPayload(queryString){
	MINIMUM_SPEED=[0x00];
	SEARCH_CRITERIA=string2ByteArray(queryString);
	NUL=[0x00];
	a=MINIMUM_SPEED;
	a=a.concat(SEARCH_CRITERIA);
	a=a.concat(NUL);
	return a;
	
}


function uuid(numbytes){
	bytes=new Array();
	for(i=0;i<numbytes;i++)
		bytes.push(Math.floor(Math.random()*127));
	return bytes;
}

function popResult(byteArray){
	
	index=pop32(byteArray);
	fileSize=pop32(byteArray);
	fileName=popString(byteArray);
	extraData=popString(byteArray);
	return new Result(index,fileSize,fileName,extraData);
}

function ResultSet(numberOfHits,byteArray){
	
	this.results=new Array(numberOfHits);
	for(i=0;i<numberOfHits;i++){
		this.results[i]=popResult(byteArray);
		//dump("\n ResultSet results["+i+"]="+this.results[i]);
	}
}

function Result(index,fileSize,fileName,extraData){
	this.index=index;
	this.fileSize=fileSize;
	this.fileName=fileName;
	this.extraData=extraData;

	dump("\n"+index);
	dump("\n"+fileSize);
	dump("\n"+fileName);
	dump("\n"+extraData);
}

